#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include "ijvm-asm.h"
#include "ijvm-util.h"

static void 
jasm_log_ap (const char *fmt, va_list ap)
{
  vfprintf (stderr, fmt, ap);
}

void
jasm_abort (const char *fmt, ...)
{
  va_list ap;

  va_start (ap, fmt);
  jasm_log_ap (fmt, ap);
  va_end (ap);
  exit (-1);
}

void
jasm_assert (int cond, const char *fmt, ...)
{
  va_list ap;

  if (!cond) {
    va_start (ap, fmt);
    jasm_log_ap (fmt, ap);
    va_end (ap);
    exit (-1);
  }
}

void
jasm_warning (const char *fmt, ...)
{
  va_list ap;

  va_start (ap, fmt);
  jasm_log_ap (fmt, ap);
  va_end (ap);
}

char *
jasm_strdup (const char *str)
{
  int len;
  char *new;

  len = strlen (str);
  new = malloc (len + 1);
  strcpy (new, str);
  return new;
}

IJVMSpec *ijvm_spec;

int
main (int argc, char *argv[])
{
  IJVMImage *image;
  JasmMethod *methods;
  JasmCPool *cpool;
  char *spec_file;
  FILE *f;
  extern int yydebug;
  int size;

  printf ("ijvm-asm\n");
  printf ("- specification=" IJVM_DATADIR "/ijvm.spec\n");

  if (argv[1] == NULL)
     {
     return (1);
     }    
  if ((strcmp (argv[1], "-v") == 0))
     {
     printf ("- version " VERSION " compiled " COMPILE_DATE " on " COMPILE_HOST "\n");
     return (1);
     }    
   

  if (getenv ("IJVM_ASM_DEBUG") != NULL)
    yydebug = 1;

  if (getenv ("IJVM_SPEC_FILE") != NULL)
    spec_file = getenv ("IJVM_SPEC_FILE");
  else {
    spec_file = IJVM_DATADIR "/ijvm.spec";
  }

  f = fopen (spec_file, "r");
  if (f == NULL)
    jasm_abort ("Couldn't read specification file `%s'.\n", spec_file);
  ijvm_spec = ijvm_spec_parse (f);
  fclose (f);

  if (argv[1] != NULL) {
    f = freopen (argv[1], "r", stdin);
    if (f == NULL)
      jasm_abort ("Couldn't open assembler file `%s'.\n", argv[1]);
  }

  if (argv[1] != NULL && argv[2] != NULL) {
    f = freopen (argv[2], "w", stdout);
    if (f == NULL)
      jasm_abort ("Couldn't open `%s' for writing.\n", argv[2]);
  }

  methods = jasm_parse ();
  cpool = jasm_cpool_make ();
  size = jasm_method_check (methods, cpool);

  image = jasm_emit (methods, cpool);
  ijvm_image_write (stdout, image);

  return 0;
}
