#include <stdlib.h>
#include <stdarg.h>
#include "ijvm-asm.h"

JasmInsn *jasm_labels = NULL;

JasmMethod *
jasm_method_make (char *name, JasmDir *dirs, JasmInsn *insns)
{
  JasmMethod *method;

  method = malloc (sizeof (JasmMethod));
  method->name = name;
  method->args = NULL;
  method->locals = NULL;
  method->dirs = dirs;
  method->defines = NULL;
  method->labels = NULL;
  method->insns = insns;
  method->next = NULL;
    
  return method;
}

JasmDir *
jasm_dir_make (JasmDirKind kind, ...)
{
  JasmDir *dir;
  va_list ap;

  dir = malloc (sizeof (JasmDir));
  dir->kind = kind;
  dir->next = NULL;
  dir->line = jasm_lex_current_line();

  va_start (ap, kind);
  switch (kind) {
  case JASM_DIR_LOCALS:
    dir->u.locals = va_arg (ap, JasmExpr*);
    break;
  case JASM_DIR_ARGS:
    dir->u.args = va_arg (ap, JasmExpr*);
    break;
  case JASM_DIR_DEFINE:
    dir->u.define.symbol = va_arg (ap, char*);
    dir->u.define.expr = va_arg (ap, JasmExpr*);
    break;
  }
  va_end (ap);
  return dir;
}

JasmInsn *
jasm_insn_make_generic (IJVMInsnTemplate *tmpl, JasmOperand *operands)
{
  JasmInsn *insn;

  insn = malloc (sizeof (JasmInsn));
  insn->kind = JASM_INSN_GENERIC;
  insn->u.generic.tmpl = tmpl;
  insn->u.generic.operands = operands;
  insn->wide = FALSE;
  insn->line = jasm_lex_current_line(); 
  insn->next = NULL;

  return insn;
}

JasmInsn *
jasm_insn_make_label (char *label)
{
  JasmInsn *insn;

  insn = malloc (sizeof (JasmInsn));
  insn->kind = JASM_INSN_LABEL;
  insn->u.label = label;
  insn->line = jasm_lex_current_line(); 
  insn->next = NULL;

  return insn;
}

JasmOperand *
jasm_operand_make (JasmExpr *expr, JasmOperand *next)
{
  JasmOperand *operand;

  operand = malloc (sizeof (JasmOperand));
  operand->expr = expr;
  operand->next = next;

  return operand;
}

JasmExpr *
jasm_expr_make (JasmExprKind kind, ...)
{
  JasmExpr *expr;
  va_list ap;

  expr = malloc (sizeof (JasmExpr));
  expr->kind = kind;
  expr->line = jasm_lex_current_line();
  expr->touch = FALSE;

  va_start (ap, kind);
  switch (kind) {
  case JASM_EXPR_INTEGER:
    expr->u.integer = va_arg (ap, int);
    break;
  case JASM_EXPR_SYMBOL:
    expr->u.symbol = va_arg (ap, char*);
    break;
  case JASM_EXPR_PLUS:
    expr->u.binop.left = va_arg (ap, JasmExpr*);
    expr->u.binop.right = va_arg (ap, JasmExpr*);
    break;
  case JASM_EXPR_MINUS:
    expr->u.binop.left = va_arg (ap, JasmExpr*);
    expr->u.binop.right = va_arg (ap, JasmExpr*);
    break;
  case JASM_EXPR_NEG:
    expr->u.neg = va_arg (ap, JasmExpr*);
    break;
  }
  va_end (ap);
  return expr;    
}

