%{

#include <stdlib.h>
#include <stdio.h>
#include "ijvm-asm.h"

#define YYDEBUG 1

int yylex(void);
void yyerror(char *msg);

static JasmMethod *program;

%}

%union {
  char *symbol;
  int integer;
  JasmMethod *method;
  JasmOperand *operand;
  JasmDir *dir;
  JasmInsn *insn;
  IJVMInsnTemplate *tmpl;
  JasmExpr *expr;
};

%token T_BIPUSH T_DUP T_GOTO T_IADD T_IAND T_IFEQ
%token T_IFLT T_IF_ICMPEQ T_IINC T_ILOAD T_INVOKEVIRTUAL 
%token T_IOR T_IRETURN T_ISTORE T_ISUB T_LDC_W T_NOP
%token T_POP T_SWAP T_NEWLINE T_METHOD T_LOCALS T_ARGS T_DEFINE

%token <symbol> T_SYMBOL T_LABEL
%token <integer> T_INTEGER
%token <tmpl> T_MNEMONIC
%type <method> program methods method
%type <dir> directives directive
%type <insn> insns insn
%type <expr> expr actual_expr top_expr
%type <operand> operands

%left '+' '-'
%left UNARY

%%

program : methods { program = $1; }

methods: 
   method methods { $1->next = $2; $$ = $1; }
 | method         { $$ = $1; }
 ;

method:
   T_METHOD T_SYMBOL directives insns
     { $$ = jasm_method_make ($2, $3, $4); }
 | T_METHOD T_SYMBOL insns
     { $$ = jasm_method_make ($2, NULL, $3); }
 ;

directives:
   directive directives { $1->next = $2; $$ = $1; }
 | directive            { $$ = $1; }
 ;

directive:
   T_ARGS top_expr          { $$ = jasm_dir_make (JASM_DIR_ARGS, $2); }
 | T_LOCALS top_expr        { $$ = jasm_dir_make (JASM_DIR_LOCALS, $2); }
 | T_DEFINE T_SYMBOL '=' top_expr
                            { $$ = jasm_dir_make (JASM_DIR_DEFINE, $2, $4); }
 ;

insns:
   insn insns { $1->next = $2; $$ = $1; }
 | insn       { $$ = $1; }
 ; 

insn:
   T_MNEMONIC operands expect_sign { $$ = jasm_insn_make_generic ($1, $2); }
 | T_MNEMONIC                      { $$ = jasm_insn_make_generic ($1, NULL); }
 | T_LABEL                         { $$ = jasm_insn_make_label ($1); }
 ;

operands:
   top_expr ',' operands { $$ = jasm_operand_make ($1, $3); }
 | top_expr              { $$ = jasm_operand_make ($1, NULL); }
 ;

top_expr: { jasm_lex_expect_operator (FALSE); } expr { $$ = $2; };

expr: actual_expr { $$ = $1; jasm_lex_expect_operator (TRUE); };

actual_expr: 
   T_INTEGER { $$ = jasm_expr_make (JASM_EXPR_INTEGER, $1); }
 | T_SYMBOL    { $$ = jasm_expr_make (JASM_EXPR_SYMBOL, $1); }
 | expr '+' expect_sign expr { $$ = jasm_expr_make (JASM_EXPR_PLUS, $1, $4); }
 | expr '-' expect_sign expr { $$ = jasm_expr_make (JASM_EXPR_MINUS, $1, $4); }
 | '-' expr %prec UNARY { $$ = jasm_expr_make (JASM_EXPR_NEG, $2); }
 | '(' expr ')' { $$ = $2; }
 ;

expect_sign: { jasm_lex_expect_operator (FALSE); };

%%

void yyerror (char *msg)
{
  extern char *yytext;

  jasm_abort ("in line %d: parse error before `%s'\n", 
	      jasm_lex_current_line (), yytext);
}

JasmMethod *
jasm_parse ()
{
  jasm_lex_expect_operator (FALSE);
  yyparse ();
  return program;
}
